-- MangosDungeonBrowser RdyCheck
-- Code by Kenuvis
-- Last Update: 16.03.2011

-- Wartezeit, wie lange der Readycheck max. dauern kann
-- bevor die Gruppe ohne ihn in den Queue zurück kehrt (in Sek.)
MDB.RdyCheckWait = 45

-------------------------------------
---- Functionhack -------------------
-------------------------------------

if MDB.Enable then

	function GetLFGProposal()
		debugprint("#GetLFGProposal")

		local name, typeID,_,_,_,_,_,_,_,texture,difficulty = GetLFGDungeonInfo(MDB.ActiveDungeonID)
		local Dungeon = MDB.Dungeons[MDB.ActiveDungeonID]
		if Dungeon == nil then return end

		-- bevorstehender Inv = 1, ansonsten nil
		local proposalExists = (Dungeon.Status == "proposal")
		-- DungeonID
		local id = MDB.ActiveDungeonID
		-- One of "HEALER", "TANK", or "DAMAGER"
		local role = Dungeon:GetPlayerInfo(UnitName("player"), "MainRole")
		-- 1 wenn Inv bereits bestätigt/abgelehnt, ansonsten nil
		local hasResponded = Dungeon:GetPlayerInfo(UnitName("player"), "Proposal")
		local totalEncounters = select(1, Dungeon:GetEncounter())
		local completedEncounters = select(2, Dungeon:GetEncounter())
		local numMembers = Dungeon:GetPlayerCount()
		-- 1 falls Leader, ansonsten nil
		local isLeader = Dungeon.IsLeader

		return proposalExists, typeID, id, name, texture, role, hasResponded, totalEncounters, completedEncounters, numMembers, isLeader
	end

	-- 1-5 für die ganze Gruppe
	function GetLFGProposalMember(PlayerNumber)
		debugprint("#GetLFGProposalMember")

		local Dungeon = MDB.Dungeons[MDB.ActiveDungeonID]

		-- 1 falls Leader, ansonsten nil
		local isLeader = Dungeon:GetPlayerInfo(PlayerNumber, "GrpLead")
		-- One of "HEALER", "TANK", or "DAMAGER"
		local role = Dungeon:GetPlayerInfo(PlayerNumber, "MainRole")

		-- Mein Proposal hat bei mir 3 Statien, nil = nicht raus, 1 = bestätigt, 0 = abgelehnt

		-- ich denk mal, ob die Einladung raus ist, dann 1, ansonsten nil
		local responded = Dungeon:GetPlayerInfo(PlayerNumber, "Proposal")
		-- ich denk mal, ob die Einladung bestätigt wurde
		local accepted = Dungeon:GetPlayerInfo(PlayerNumber, "Proposal")

		local level = Dungeon:GetPlayerInfo(PlayerNumber, "Level")
		local class = Dungeon:GetPlayerInfo(PlayerNumber, "Class")

		return isLeader, role, level, responded, accepted, name, class
	end

	-- EncounterNumber kommt von GetLFGProposal()
	-- Dazu müsste man eine Datenbank hinterlegen, ein Kill-Hook und ein Dungeonreset-Hook
	function GetLFGProposalEncounter(EncounterNumber)
		debugprint("#GetLFGProposalEncounter")

		local Dungeon = MDB.Dungeons[MDB.ActiveDungeonID]

		local bossName = select(1, Dungeon:GetEncounter(EncounterNumber))
		local texture = ""
		local isKilled = select(2, Dungeon:GetEncounter(EncounterNumber))

		return bossName, texture, isKilled
	end

	function AcceptProposal()
		MDB.Dungeons[MDB.ActiveDungeonID]:SetPlayerInfo(UnitName("player"), "Proposal", 1)
		MDB.Dungeons[MDB.ActiveDungeonID]:SendGroup("rdycheck", "accept,"..UnitName("player"))
		LFDFrame_OnEvent(nil, "LFG_PROPOSAL_UPDATE")
		MDB.Dungeons[MDB.ActiveDungeonID]:CheckFullProposal()
	end

	function RejectProposal()
		MDB.Dungeons[MDB.ActiveDungeonID]:SetPlayerInfo(UnitName("player"), "Proposal", nil)
		MDB.Dungeons[MDB.ActiveDungeonID]:SendGroup("rdycheck", "reject,"..UnitName("player"))
		MDB_SystemMessage(ERR_LFG_PROPOSAL_DECLINED_SELF)
		LFDFrame_OnEvent(nil, "LFG_PROPOSAL_FAILED")
		LeaveLFG()
	end

end

-------------------------------------
---- Interaction --------------------
-------------------------------------

function MDB_Event:rdycheck(Message, Sender)
	if Message[1] == "start" then
		self:StartRdyCheck()

	elseif Message[1] == "reject" then
		MDB_SystemMessage(ERR_LFG_PROPOSAL_DECLINED_PARTY)
		self:RdyCheckFinish(false)

	elseif Message[1] == "accept" then
		self:SetPlayerInfo(Sender, "Proposal", 1)
		LFDFrame_OnEvent(nil, "LFG_PROPOSAL_UPDATE")
		self:CheckFullProposal()

	elseif Message[1] == "timeout" then
		self:RdyCheckFinish(false)
	end
end

-------------------------------------
---- interne Operation --------------
-------------------------------------

function MDB_Prototype:StartRdyCheck()
	self.Status = "proposal"

	LFDFrame_OnEvent(nil, "LFG_PROPOSAL_SHOW")

	if self.IsLeader then
		self:SendGroup("rdycheck", "start")
		MDB_StopUhr:Start("rdycheck", MDB.RdyCheckWait, function() self:RdyCheckFinish("timeout") end)
	else
		-- Wenn der Leader irgendein Fehler bekommt, dass die Mitglieder nicht sinnlos in der Luft hngen
		MDB_StopUhr:Start("rdycheck", MDB.RdyCheckWait+5, function() self:RdyCheckFinish(false) end)
	end
end

function MDB_Prototype:CheckFullProposal()
	local Accept = 1
	for a=1,self:GetPlayerCount() do
		Accept = self:GetPlayerInfo(a, "Proposal")
		if not Accept then
			return
		end
	end

	-- alle haben nun bestätigt
	self:RdyCheckFinish(true)
end

function MDB_Prototype:RdyCheckFinish(complete)
	MDB_StopUhr:Stop("rdycheck")
	-- egal wie es ausgeht, der rdycheck ist beendet
	self.Status = "queue"

	if complete == true then
		LFDFrame_OnEvent(nil, "LFG_PROPOSAL_SUCCEEDED")
		self:StartDungeon()

	elseif complete == "timeout" then
		for a=1,self:GetPlayerCount() do
			if not self:GetPlayerInfo(a, "Proposal") then
				local name = self:GetPlayerInfo(a, "Name")
				-- Spieler hat nicht besttigt
				if name == UnitName("player") then
					LeaveLFG()
				else
					self:Send("timeout", "proposal", name)
					self:RemovePlayerFromDungeonQueue(name)
				end
			end
		end
		self:SendGroup("rdycheck", "timeout")
		self:RdyCheckFinish(false)

	else
		--error("complete = false",3)
		self.AcceptInvite = nil

		for a=1,self:GetPlayerCount() do
			self:SetPlayerInfo(a, "Proposal", nil)
		end

		LFDFrame_OnEvent(nil, "LFG_PROPOSAL_FAILED")
		MDB_SystemMessage(ERR_LFG_PROPOSAL_FAILED)
	end
end
