-- MangosDungeonBrowser RoleCheck
-- Code by Kenuvis
-- Last Update: 15.02.2011

--[[
Wie lange bleibt der Rolecheck aktiv (in Sekunden)
]]
MDB.RoleCheckWait = 45

local NumRoleCheck = 0

-------------------------------------
---- Functionhack -------------------
-------------------------------------

if MDB.Enable then

	function CompleteLFGRoleCheck(accept)
		debugprint("#CompleteLFGRoleCheck")

		if not accept then
			MDB_RCGroupSend("fail,incomplete")
			MDB_Prototype:FinishRoleCheck(false)
			return
		end

		MDB_RCGroupSend("roles,"..MDB_Prototype:ConvertPlayerInfo("myself"))

		if not IsPartyLeader() then
			local i = GetPartyLeaderIndex()
			MDB_Prototype:Send("rolecheck", "complete", UnitName("party"..i))
		end

		local me = UnitName("player")
		for id in pairs(MDB.Dungeons) do
			MDB.Dungeons[id]:SetPlayerInfo(me, "Lead", MDB_Roles.Leader)
			MDB.Dungeons[id]:SetPlayerInfo(me, "Tank", MDB_Roles.Tank)
			MDB.Dungeons[id]:SetPlayerInfo(me, "Heal", MDB_Roles.Healer)
			MDB.Dungeons[id]:SetPlayerInfo(me, "DPS", MDB_Roles.DPS)
			--MDB.Dungeons[id].enabled = true
		end

		LFDFrame_OnEvent(nil, "LFG_ROLE_CHECK_HIDE")
	end

	function GetLFGRoleUpdate()
		debugprint("#GetLFGRoleUpdate")

		local Dungeon = MDB.Dungeons[MDB.ActiveDungeonID]
		if not Dungeon then return end

		local inProgress = (Dungeon.Status == "rolecheck")
		local slots = 0
		table.foreach(MDB.Dungeons, function(id)
			slots = slots + 1
			MDB.Dungeons[id].slot = slots
		end)
		local members = GetNumPartyMembers()+1

		return inProgress, slots, members
	end

	function GetLFGRoleUpdateSlot(slot)
		debugprint("#GetLFGRoleUpdateSlot")

		local dungeonID
		for id in pairs(MDB.Dungeons) do
			if MDB.Dungeons[id].slot == slot then
				dungeonID = id
				break
			end
		end

		if not dungeonID then return end

		local dungeonType = select(LFG_RETURN_VALUES.typeID, GetLFGDungeonInfo(dungeonID))

		return dungeonType, dungeonID
	end

end

-------------------------------------
---- Interaction --------------------
-------------------------------------

function MDB_Event:rolecheck(Message, Sender)
	if Message[1] == "start" then
		table.remove(Message, 1)

		local _,Leader, Tank, Healer, DPS = MDB_Prototype:ConvertPlayerInfo(Message[1],true)
		table.remove(Message, 1)

		local valid = false
		for _,id in pairs(Message) do
			if MDB.CustomLockList[id] and not MDB_IsDebug() then
				MDB_RCGroupSend("invaliddungeon,"..id)
			else
				-- min. eine Dungeon ist zugnglich mit der Gruppe
				valid = true
				MDB_InitDungeon(id, false)
				MDB.ActiveDungeonID = id
				MDB.Dungeons[id].IsLeader = false
				MDB.Dungeons[id].IsMember = true
				MDB.Dungeons[id]:AddPlayerToDungeonQueue(Sender, Leader, Tank, Healer, DPS)
			end
		end

		if not valid then
			MDB_RCGroupSend("fail,valid")
			MDB_SystemMessage(ERR_LFG_ROLE_CHECK_FAILED_NOT_VIABLE)
			MDB_Prototype:FinishRoleCheck(false)
			return
		end

		LFDFrame_OnEvent(nil, "LFG_ROLE_CHECK_SHOW")

		if not MDB.ChannelID or MDB.ChannelID == 0 then
			JoinChannelByName(MDB.Channel, MDB.ChannelPassword)
			MDB.ChannelID = GetChannelName(MDB.Channel)
		end

	elseif Message[1] == "invaliddungeon" then
		MDB_RemoveDungeon(Message[2])
		MDB.CustomLockList[Message[2]] = 1

	elseif Message[1] == "complete" then
		local Tank, Heal, DPS
		for id in pairs(MDB.Dungeons) do
			Tank = MDB.Dungeons[id]:GetPlayerInfo(Sender, "Tank")
			Heal = MDB.Dungeons[id]:GetPlayerInfo(Sender, "Heal")
			DPS = MDB.Dungeons[id]:GetPlayerInfo(Sender, "DPS")

			if not Tank and not Heal and not DPS then
				MDB_RCGroupSend("fail,role,"..Sender)
				MDB_SystemMessage(ERR_LFG_PLAYER_DECLINED_ROLE_CHECK, Sender)
				MDB_Prototype:FinishRoleCheck(false)
				return
			elseif not MDB.Dungeons[id]:RoleArrange() then
				MDB_RCGroupSend("fail,restart")
				MDB_SystemMessage(ERR_LFG_ROLE_CHECK_FAILED_NOT_VIABLE)
				LFDFrame_OnEvent(nil, "LFG_ROLE_CHECK_SHOW")
				return
			else

			end
			break
		end

		NumRoleCheck = NumRoleCheck - 1
		if NumRoleCheck == 0 then
			MDB_Prototype:FinishRoleCheck(true)
		end

	elseif Message[1] == "fail" then

		if Message[2] == "incomplete" then
			MDB_SystemMessage(ERR_LFG_ROLE_CHECK_FAILED)
		elseif Message[2] == "role" then
			MDB_SystemMessage(ERR_LFG_PLAYER_DECLINED_ROLE_CHECK, Message[3])
		elseif Message[2] == "aborted" then
			MDB_SystemMessage(ERR_LFG_ROLE_CHECK_ABORTED)
		elseif Message[2] == "valid" then
			MDB_SystemMessage(ERR_LFG_ROLE_CHECK_FAILED_NOT_VIABLE)
		elseif Message[2] == "restart" then
			MDB_SystemMessage(ERR_LFG_ROLE_CHECK_FAILED_NOT_VIABLE)
			LFDFrame_OnEvent(nil, "LFG_ROLE_CHECK_SHOW")
			return
		end
		MDB_Prototype:FinishRoleCheck(false)

	elseif Message[1] == "roles" then
		local name,leader,tank,healer,dps = MDB_Prototype:ConvertPlayerInfo(Message[2], true)
		for id in pairs(MDB.Dungeons) do
			MDB.Dungeons[id]:SetPlayerInfo(Sender, "Lead", leader)
			MDB.Dungeons[id]:SetPlayerInfo(Sender, "Tank", tank)
			MDB.Dungeons[id]:SetPlayerInfo(Sender, "Heal", heal)
			MDB.Dungeons[id]:SetPlayerInfo(Sender, "DPS", dps)
		end

		LFGEventFrame_OnEvent(nil, "LFG_ROLE_CHECK_ROLE_CHOSEN", name, (tank == 1), (heal == 1), (dps == 1))

	elseif Message[1] == "finish" then
		MDB_QueueUpdate()
		MDB_SystemMessage(ERR_LFG_JOINED_QUEUE)

		for id in pairs(MDB.Dungeons) do
			MDB.Dungeons[id].enabled = true
		end
	end
end

-------------------------------------
---- interne Operation --------------
-------------------------------------

function MDB_Prototype:StartRoleCheck(SendString)
	-- raids knnen generell nicht beitreten
	if GetNumRaidMembers() > 0 or GetNumPartyMembers() > 5 then
		MDB_SystemMessage(ERR_LFG_TOO_MANY_MEMBERS)
		return
	end

	NumRoleCheck = GetNumPartyMembers()
	for a = 1, NumRoleCheck do
		local name = UnitName("party"..a)
		table.foreach(MDB.Dungeons, function(id)
			MDB.Dungeons[id]:AddPlayerToDungeonQueue(name)
			MDB.Dungeons[id].Status = "rolecheck"
			MDB.Dungeons[id].enabled = false
			MDB.Dungeons[id]:SetPlayerInfo(UnitName("player"), "GrpLead", 1)
		end)
		MDB_Prototype:Send("rolecheck", "start,"..SendString, name)
	end

	MDB_SystemMessage(ERR_LFG_ROLE_CHECK_INITIATED)
	MDB_StopUhr:Start("rolecheck", MDB.RoleCheckWait, function() self:FinishRoleCheck("timeout") end)
end

function MDB_Prototype:FinishRoleCheck(Status)
	LFDFrame_OnEvent(nil, "LFG_ROLE_CHECK_HIDE")
	MDB_StopUhr:Stop("rolecheck")

	if Status == "timeout" then
		MDB_SystemMessage(ERR_LFG_ROLE_CHECK_FAILED_TIMEOUT)
		self:FinishRoleCheck(false)
	elseif Status == false then
		MDB.Dungeons = {}
		MDB.ActiveDungeonID = 0
	elseif Status == true then
		local DungeonForQueue = {}
		table.foreach(MDB.Dungeons, function(id)
			MDB.Dungeons[id].enabled = true
			MDB.Dungeons[id].Status = "queue"
			table.insert(DungeonForQueue, id)
		end)

		local SendString = MDB.Dungeons[DungeonForQueue[1]]:ConvertPlayerList()
		SendString = SendString..","..table.concat(DungeonForQueue, ",")

		MDB_Prototype:Send("join", SendString)
		MDB_QueueUpdate()
		MDB_SystemMessage(ERR_LFG_JOINED_QUEUE)
		MDB_RCGroupSend("finish")
		MDB.Dungeons[MDB.ActiveDungeonID]:TriggerGroupUpdate()
	end
end

function MDB_RCGroupSend(msg)
	for a = 1, GetNumPartyMembers() do
		MDB_Prototype:Send("rolecheck", msg, UnitName("party"..a))
	end
end
